/* Copyright 2017-2022 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

static const char *g_string_initializer = "hello";

int
main ()
{
   int integers[10];
   const char *strings[10];

   for (auto &i : integers)
     i = 0;

   for (auto &i : strings)
     i = g_string_initializer;

  return 0;			// break here
}
